
' =========================================================================
'
'	File...... TSL1401_scan.bs2
'	Purpose... Image capture and processing demo using the TSL1401-DB
'	Author.... Phil Pilgrim, Bueno Systems, Inc.
'	E-mail....
'	Started... 11 July 2007
'	Updated...
'
'	{$STAMP BS2}
'	{$PBASIC 2.5}
'
' =========================================================================

' -----[ Program Description ]---------------------------------------------

' This program demonstrates image capture and processing using the
' TSL1401-DB (Parallax p/n 28317). It continuously acquires and displays
' images from the TSL1401R sensor chip. It then locates both left and right
' bright edges, displaying them graphically using DEBUG. Finally it
' computes both the area and extent of the object found.

CON
    _clkmode = xtal1 + pll16x
    _xinfreq = 5_000_000

OBJ

  SPI     :       "SPI_Asm"                               ''The Standalone SPI Assembly engine
  Debug   :       "FullDuplexSerialPlus"                  ''Used for Debug

CON

' -----[ I/O Definitions ]-------------------------------------------------

  ao = 0
  si = 1
  clk= 2

' -----[ Constants ]-------------------------------------------------------

  DRK   = 0	'Value assignd to "which" for dark pixels.
  BRT   = 1	'Value assigned to "which" for bright pixels.
  FWD   = 0	'Value assigned to "dir" for left-to-right search.
  BKWD  = 1     'Value assigned to "dir" for right-to-left search.


VAR


byte  pixel[128]
long  exp
byte  lptr
byte  rptr
byte  count
byte  which
byte  dir
byte  found

PUB Main

Debug.start(31, 30, 0, 57600)
SPI.start(5,1)             '' Initialize SPI Engine with Clock Delay of 5 (500nS) and Clock State of 1

waitcnt(clkfreq + cnt)

exp := clkfreq/1000 *8	'Set exposure time to 8000 uSec (1/120th sec).

DispHdr	              'Display the pixel location header.

Repeat  10

  GetPix



  DispPix

  lptr := 0              'Initialize parameters for find.
  rptr := 127
  which := BRT
  dir := FWD
  found := 1
  FindEdge               'Find first dark-to-light edge going L->R.
  dir := BKWD            'Switch directions.
  FindEdge              'Find first dark-to-light edge going L<-R.

'  CLREOL

  IF found        	'Both edges found?
        CRSRX(lptr - 1)
        Debug.str(string("_|"))                                        'Yes: Display left edge.
        CRSRX(rptr)
        Debug.str(string("|_"))	                                        '	Display right edge.
        CountPix	                                                        '	Compute area (light pixel count).
        CR
        CR
        Debug.str(string("Area = "))
        Debug.dec(count)
        Debug.str(string("Extent = "))
        Debug.dec(rptr - lptr + 1)
        CLREOL
  ELSE	              'No: Display failure message.
        CR
        CR
        Debug.str(string("No object found = "))
        CLREOL


' -----[ Subroutines ]-----------------------------------------------------

' -----[ GetPix ]----------------------------------------------------------
' Acquire 128 thresholded pixels from sensor chip.
' exp is the exposure time in microseconds.

PUB GetPix | i

  dira[si]~~
  outa[si]~~
  repeat i from 0 to 127
    pixel[i] := SPI.SHIFTIN(ao, clk, SPI#LSBPRE, 1)
  !outa[si]
  waitcnt(cnt + exp)
  outa[si]~~
  repeat i from 0 to 127
    pixel[i] := SPI.SHIFTIN(ao, clk, SPI#LSBPRE, 1)
  !outa[si]

PUB DispHdr  | i

'  Home

  Repeat i from 0 to 9
         Debug.dec(i)	                                'Display tens digits.
         DEBUG.str(string("         "))
  Repeat i from 0 to 2
         Debug.dec(i)	                                'Display tens digits.
         DEBUG.str(string("         "))

  CR

  Repeat i from 0 to 127	                                'Display ones digits.
        DEBUG.DEC(i // 10)
  CR

PUB DispPix  | i                                         ' Display 128 pixels: light pixels as "1"; dark, as "_".

Repeat i from 0 to 127
  IF pixel[i]
    DEBUG.tx("1")
  ELSE
    DEBUG.tx(".")
CR

PUB FindEdge

' Find the first edge within the range lptr through rptr, in the direction
' given by dir and of the type indicated by which (0 = light-to-dark; ' 1 = dark-to-light).

  which := 1 - which	        'Look for opposite kind of pixel first.
  FindPix
  which := 1 - which	        'Then look for desired pixel,

PUB FindPix | i

' Find the first pixel within the range lptr through rptr, in the direction
' given by dir and of the type indicated by which (0 = dark; 1 = light).

  IF (found == 1 AND lptr =< rptr AND rptr =< 127)      'Still looking & within bounds?


        IF (dir == FWD)	                                ' Yes: Search left-to-right?
           repeat lptr from lptr to rptr	        '	Yes: Loop forward.
              IF (pixel[lptr] == which)
                 Return                                  '	Return on match.

        ELSE
            repeat rptr from rptr to lptr	         '	No: Loop backward.
               IF (pixel[rptr] == which)
                 Return                                  '	Return on match.
  found := 0	                                        'Didn't look or nothing found.

' -----[CountPix]----------------------------------------------------------

' Count pixels within the range lptr through rptr, of the type indicated by
' which (0 = dark; 1 = light).

PUB CountPix  | i

  count := 0	                                                                'Initialize count.
  IF (lptr =< rptr AND rptr =< 127)                                             'Valid range?
    repeat i from lptr to rptr	                                                ' Yes: Loop over desired range.
      IF (pixel[i] == which)                                                   'Add to count when pixel matches.
       count++

PUB CR

  debug.tx(13)
PUB CS
{{Clear screen and place cursor at top-left.}}

  debug.tx(16)

PUB CLREOL
{{Clear line from cursor to end of line.}}

  debug.tx(11)


PUB Home
{{Send cursor to home position (top-left).}}

  debug.tx(1)

PUB CRSRXY(x, y)
{{Position cursor at column x, row y (from top-left).}}

  debug.tx(2)
  debug.tx(x)
  debug.tx(y)


PUB CRSRX(x)
{{Position cursor at column x of current row.}}
  debug.tx(2)
  debug.tx(x)

PUB CRSRY(y)
{{Position cursor at row y of current column.}}
  debug.tx(2)
  debug.tx(y)

PUB LineFeed
{{Send cursor down to next line.}}

  debug.tx(10)

PUB MoveLeft(x)
{{Move cursor left x characters.}}

  repeat x
    debug.tx(3)


